from lib.config import *


def convert_scad(val):
    if isinstance(val, Distance):
        return val.as_mm()
    elif isinstance(val, Angle):
        return val.as_degrees()
    elif isinstance(val, (list, tuple)):
        res = list()
        for item in val:
            res.append(convert_scad(item))
        return res
    else:
        return val


def _recur_scad_config(fd, base, variables):

    if isinstance(variables, dict):
        for key, item in variables.items():
            if base == "":
                newbase = key
            else:
                newbase = base + "_" + key

            _recur_scad_config(fd, newbase, item)

            if base == "":
                fd.write("\n")

    elif isinstance(variables, (list, tuple)):

        fd.write("{} = [{}];\n".format(base, ",".join(
            str(convert_scad(x)) for x in variables)))

    elif isinstance(variables, (int, float)):
        fd.write("{} = {};\n".format(base, convert_scad(variables)))

    else:
        raise Exception("unknown data", variables)


def generate_scad_config(filename, variables):
    print("generating scad file: " + filename)
    with open(filename, "w") as fd:
        fd.write("// rendered by script, do not edit manually!\n")
        _recur_scad_config(fd, "", variables)


def generate_scad_joint_pos(fd, key, joint):
    fd.write("module " + key + "_joint_pos()")
    fd.write(" translate({})".format(convert_scad(joint['pos'])))
    fd.write(" rotate({})".format(convert_scad(joint['rot'])))
    fd.write("{\n")
    module_args = joint['scad']['args']
    module_args = ",".join([str(convert_scad(i)) for i in module_args])
    fd.write("  {}({});\n".format(joint['scad']['module'], module_args))
    fd.write("  children();")
    fd.write("\n}\n")
    fd.write("\n")


def generate_scad_leg_pos(filename, legs):
    with open(filename, "w") as fd:
        fd.write("include <leg_modules.scad>;\n")
        for leg_key, leg in legs.items():
            for joint_key, joint in leg['joints'].items():
                generate_scad_joint_pos(fd, leg_key + "_" + joint_key, joint)

            fd.write("module {}_leg()".format(leg_key))
            fd.write("translate({})".format(convert_scad(leg['pos'])))
            fd.write("rotate({})".format(convert_scad(leg['rot'])))
            fd.write("mirror({})".format(leg['mirror']))
            fd.write("{\n")

            for key in leg['joint_chain']:
                fd.write(" {}_{}_joint_pos()".format(leg_key, key))
            fd.write(";\n}\n")

        fd.write("module legs(){\n")
        for leg_key in legs.keys():
            fd.write(" {}_leg();\n".format(leg_key))
        fd.write("}\n")

        fd.write("module leg_pos(){\n")
        for leg_key, leg in legs.items():
            fd.write("translate({})".format(convert_scad(leg['pos'])))
            fd.write("rotate({})".format(convert_scad(leg['rot'])))
            fd.write("mirror({})".format(leg['mirror']))
            fd.write("children();\n")

        fd.write("}\n")
